@@ -13,7 +13,7 @@ from ._async.computer_use import (
     ScreenshotRegion,
 )
 from ._async.daytona import AsyncDaytona
-from ._async.sandbox import AsyncSandbox
+from ._async.sandbox import AsyncPaginatedSandboxes, AsyncSandbox
 from ._sync.daytona import Daytona
 from ._sync.sandbox import Sandbox
 from .common.charts import (
@@ -34,10 +34,11 @@ from .common.daytona import (
     DaytonaConfig,
 )
 from .common.errors import DaytonaError
-from .common.filesystem import FileUpload
+from .common.filesystem import FileDownloadRequest, FileDownloadResponse, FileUpload
 from .common.image import Image
 from .common.lsp_server import LspLanguageId
 from .common.process import CodeRunParams, SessionExecuteRequest
+from .common.pty import PtySize
 from .common.sandbox import Resources
 from .common.snapshot import CreateSnapshotParams
 from .common.volume import VolumeMount
@@ -62,10 +63,13 @@ __all__ = [
     "PieChart",
     "BoxAndWhiskerChart",
     "CompositeChart",
+    "FileDownloadRequest",
+    "FileDownloadResponse",
     "FileUpload",
     "VolumeMount",
     "AsyncDaytona",
     "AsyncSandbox",
+    "AsyncPaginatedSandboxes",
     "AsyncComputerUse",
     "AsyncMouse",
     "AsyncKeyboard",
@@ -78,4 +82,5 @@ __all__ = [
     "CreateSandboxFromImageParams",
     "CreateSandboxFromSnapshotParams",
     "CreateSnapshotParams",
+    "PtySize",
 ]
