@@ -6,7 +6,7 @@ import json
 import time
 import warnings
 from importlib.metadata import version
-from typing import Callable, Dict, List, Optional, Union, overload
+from typing import Callable, Dict, Optional, Union, overload
 
 from daytona_api_client_async import (
     ApiClient,
@@ -26,6 +26,7 @@ from .._utils.enum import to_enum
 from .._utils.errors import DaytonaError, intercept_errors
 from .._utils.stream import process_streaming_response
 from .._utils.timeout import with_timeout
+from ..code_toolbox.sandbox_js_code_toolbox import SandboxJsCodeToolbox
 from ..code_toolbox.sandbox_python_code_toolbox import SandboxPythonCodeToolbox
 from ..code_toolbox.sandbox_ts_code_toolbox import SandboxTsCodeToolbox
 from ..common.daytona import (
@@ -35,7 +36,7 @@ from ..common.daytona import (
     DaytonaConfig,
     Image,
 )
-from .sandbox import AsyncSandbox
+from .sandbox import AsyncPaginatedSandboxes, AsyncSandbox
 from .snapshot import AsyncSnapshotService
 from .volume import AsyncVolumeService
 
@@ -165,7 +166,7 @@ class AsyncDaytona:
         # Get SDK version dynamically
         try:
             sdk_version = None
-            for pkg_name in ["daytona_sdk", "daytona"]:
+            for pkg_name in ["daytona", "daytona_sdk"]:
                 try:
                     sdk_version = version(pkg_name)
                     break
@@ -486,7 +487,9 @@ class AsyncDaytona:
         language = enum_language
 
         match language:
-            case CodeLanguage.JAVASCRIPT | CodeLanguage.TYPESCRIPT:
+            case CodeLanguage.JAVASCRIPT:
+                return SandboxJsCodeToolbox()
+            case CodeLanguage.TYPESCRIPT:
                 return SandboxTsCodeToolbox()
             case CodeLanguage.PYTHON:
                 return SandboxPythonCodeToolbox()
@@ -529,7 +532,7 @@ class AsyncDaytona:
         Example:
             ```python
             sandbox = await daytona.get("my-sandbox-id")
-            print(sandbox.status)
+            print(sandbox.state)
             ```
         """
         if not sandbox_id:
@@ -569,39 +572,54 @@ class AsyncDaytona:
         """
         if sandbox_id:
             return await self.get(sandbox_id)
-        sandboxes = await self.list(labels)
+        sandboxes = await self.list(labels, page=1, limit=1)
         if len(sandboxes) == 0:
             raise DaytonaError(f"No sandbox found with labels {labels}")
         return sandboxes[0]
 
     @intercept_errors(message_prefix="Failed to list sandboxes: ")
-    async def list(self, labels: Optional[Dict[str, str]] = None) -> List[AsyncSandbox]:
-        """Lists Sandboxes filtered by labels.
+    async def list(
+        self, labels: Optional[Dict[str, str]] = None, page: Optional[int] = None, limit: Optional[int] = None
+    ) -> AsyncPaginatedSandboxes:
+        """Returns paginated list of Sandboxes filtered by labels.
 
         Args:
             labels (Optional[Dict[str, str]]): Labels to filter Sandboxes.
+            page (Optional[int]): Page number for pagination (starting from 1).
+            limit (Optional[int]): Maximum number of items per page.
 
         Returns:
-            List[Sandbox]: List of Sandbox instances that match the labels.
+            AsyncPaginatedSandboxes: Paginated list of Sandbox instances that match the labels.
 
         Example:
             ```python
-            sandboxes = await daytona.list(labels={"my-label": "my-value"})
-            for sandbox in sandboxes:
-                print(f"{sandbox.id}: {sandbox.status}")
+            result = await daytona.list(labels={"my-label": "my-value"}, page=2, limit=10)
+            for sandbox in result.items:
+                print(f"{sandbox.id}: {sandbox.state}")
             ```
         """
-        sandboxes = await self._sandbox_api.list_sandboxes(labels=json.dumps(labels))
+        if page is not None and page < 1:
+            raise DaytonaError("page must be a positive integer")
 
-        return [
-            AsyncSandbox(
-                sandbox,
-                self._sandbox_api,
-                self._toolbox_api,
-                self._get_code_toolbox(self._validate_language_label(sandbox.labels.get("code-toolbox-language"))),
-            )
-            for sandbox in sandboxes
-        ]
+        if limit is not None and limit < 1:
+            raise DaytonaError("limit must be a positive integer")
+
+        response = await self._sandbox_api.list_sandboxes_paginated(labels=json.dumps(labels), page=page, limit=limit)
+
+        return AsyncPaginatedSandboxes(
+            items=[
+                AsyncSandbox(
+                    sandbox,
+                    self._sandbox_api,
+                    self._toolbox_api,
+                    self._get_code_toolbox(self._validate_language_label(sandbox.labels.get("code-toolbox-language"))),
+                )
+                for sandbox in response.items
+            ],
+            total=response.total,
+            page=response.page,
+            total_pages=response.total_pages,
+        )
 
     def _validate_language_label(self, language: Optional[str]) -> CodeLanguage:
         """Validates and normalizes the language label.
