@@ -4,16 +4,24 @@
 import io
 import os
 from contextlib import ExitStack
-from typing import Awaitable, Callable, List, Union, overload
+from typing import List, Union, overload
 
 import aiofiles
 import aiofiles.os
 import httpx
-from daytona_api_client_async import FileInfo, Match, ReplaceRequest, ReplaceResult, SearchFilesResponse, ToolboxApi
+from daytona_api_client_async import (
+    DownloadFiles,
+    FileInfo,
+    Match,
+    ReplaceRequest,
+    ReplaceResult,
+    SearchFilesResponse,
+    ToolboxApi,
+)
+from multipart import MultipartSegment, PushMultipartParser, parse_options_header
 
-from .._utils.errors import intercept_errors
-from .._utils.path import prefix_relative_path
-from ..common.filesystem import FileUpload
+from .._utils.errors import DaytonaError, intercept_errors
+from ..common.filesystem import FileDownloadRequest, FileDownloadResponse, FileUpload
 
 
 class AsyncFileSystem:
@@ -27,18 +35,15 @@ class AsyncFileSystem:
         self,
         sandbox_id: str,
         toolbox_api: ToolboxApi,
-        get_root_dir: Callable[[], Awaitable[str]],
     ):
         """Initializes a new FileSystem instance.
 
         Args:
             sandbox_id (str): The Sandbox ID.
             toolbox_api (ToolboxApi): API client for Sandbox operations.
-            get_root_dir (Callable[[], str]): A function to get the default root directory of the Sandbox.
         """
         self._sandbox_id = sandbox_id
         self._toolbox_api = toolbox_api
-        self._get_root_dir = get_root_dir
 
     @intercept_errors(message_prefix="Failed to create folder: ")
     async def create_folder(self, path: str, mode: str) -> None:
@@ -47,7 +52,7 @@ class AsyncFileSystem:
 
         Args:
             path (str): Path where the folder should be created. Relative paths are resolved based
-            on the user's root directory.
+            on the sandbox working directory.
             mode (str): Folder permissions in octal format (e.g., "755" for rwxr-xr-x).
 
         Example:
@@ -59,7 +64,6 @@ class AsyncFileSystem:
             await sandbox.fs.create_folder("workspace/secrets", "700")
             ```
         """
-        path = prefix_relative_path(await self._get_root_dir(), path)
         print(f"Creating folder {path} with mode {mode}")
         await self._toolbox_api.create_folder(
             self._sandbox_id,
@@ -68,11 +72,12 @@ class AsyncFileSystem:
         )
 
     @intercept_errors(message_prefix="Failed to delete file: ")
-    async def delete_file(self, path: str) -> None:
+    async def delete_file(self, path: str, recursive: bool = False) -> None:
         """Deletes a file from the Sandbox.
 
         Args:
-            path (str): Absolute path to the file to delete.
+            path (str): Path to the file to delete. Relative paths are resolved based on the sandbox working directory.
+            recursive (bool): If the file is a directory, this must be true to delete it.
 
         Example:
             ```python
@@ -80,9 +85,7 @@ class AsyncFileSystem:
             await sandbox.fs.delete_file("workspace/data/old_file.txt")
             ```
         """
-        await self._toolbox_api.delete_file(
-            self._sandbox_id, path=prefix_relative_path(await self._get_root_dir(), path)
-        )
+        await self._toolbox_api.delete_file(self._sandbox_id, path=path, recursive=recursive)
 
     @overload
     async def download_file(self, remote_path: str, timeout: int = 30 * 60) -> bytes:
@@ -91,8 +94,8 @@ class AsyncFileSystem:
         It can only be used for smaller files.
 
         Args:
-            remote_path (str): Path to the file in the Sandbox. Relative paths are resolved based on the user's
-            root directory.
+            remote_path (str): Path to the file in the Sandbox. Relative paths are resolved based
+            on the sandbox working directory.
             timeout (int): Timeout for the download operation in seconds. 0 means no timeout. Default is 30 minutes.
 
         Returns:
@@ -117,8 +120,8 @@ class AsyncFileSystem:
         This method is useful when you want to download larger files that may not fit into memory.
 
         Args:
-            remote_path (str): Path to the file in the Sandbox. Relative paths are resolved based on the user's
-            root directory.
+            remote_path (str): Path to the file in the Sandbox. Relative paths are resolved based
+            on the sandbox working directory.
             local_path (str): Path to save the file locally.
             timeout (int): Timeout for the download operation in seconds. 0 means no timeout. Default is 30 minutes.
 
@@ -136,19 +139,73 @@ class AsyncFileSystem:
         if len(args) == 1 or (len(args) == 2 and isinstance(args[1], int)):
             remote_path = args[0]
             timeout = args[1] if len(args) == 2 else 30 * 60
-            return await self._toolbox_api.download_file(
-                self._sandbox_id,
-                path=prefix_relative_path(await self._get_root_dir(), remote_path),
-                _request_timeout=timeout or None,
-            )
+            response = (await self.download_files([FileDownloadRequest(source=remote_path)], timeout=timeout))[0]
+            if response.error:
+                raise DaytonaError(response.error)
+            return response.result
 
         remote_path = args[0]
         local_path = args[1]
         timeout = args[2] if len(args) == 3 else 30 * 60
         # pylint: disable=protected-access
-        method, url, headers, *_ = self._toolbox_api._download_file_serialize(
+        response = (
+            await self.download_files(
+                [FileDownloadRequest(source=remote_path, destination=local_path)], timeout=timeout
+            )
+        )[0]
+        if response.error:
+            raise DaytonaError(response.error)
+        return None
+
+    @intercept_errors(message_prefix="Failed to download files: ")
+    async def download_files(
+        self, files: List[FileDownloadRequest], timeout: int = 30 * 60
+    ) -> List[FileDownloadResponse]:
+        """Downloads multiple files from the Sandbox. If the files already exist locally, they will be overwritten.
+
+        Args:
+            files (List[FileDownloadRequest]): List of files to download.
+            timeout (int): Timeout for the download operation in seconds. 0 means no timeout. Default is 30 minutes.
+
+        Returns:
+            List[FileDownloadResponse]: List of download results.
+
+        Raises:
+            Exception: Only if the request itself fails (network issues, invalid request/response, etc.). Individual
+            file download errors are returned in the `FileDownloadResponse.error` field.
+
+        Example:
+            ```python
+            # Download multiple files
+            results = await sandbox.fs.download_files([
+                FileDownloadRequest(source="tmp/data.json"),
+                FileDownloadRequest(source="tmp/config.json", destination="local_config.json")
+            ])
+            for result in results:
+                if result.error:
+                    print(f"Error downloading {result.source}: {result.error}")
+                elif result.result:
+                    print(f"Downloaded {result.source} to {result.result}")
+            ```
+        """
+        if not files:
+            return []
+
+        class FileMeta:
+            def __init__(self, dst: str | None):
+                self.dst = dst
+                self.error: str | None = None
+                self.result: str | bytes | None = None
+
+        src_file_meta_dict = {}
+        file_writers = []
+        for f in files:
+            src_file_meta_dict[f.source] = FileMeta(dst=f.destination)
+
+        # pylint: disable=protected-access
+        method, url, headers, body, *_ = self._toolbox_api._download_files_serialize(
             self._sandbox_id,
-            path=prefix_relative_path(await self._get_root_dir(), remote_path),
+            download_files=DownloadFiles(paths=list(src_file_meta_dict.keys())),
             x_daytona_organization_id=None,
             _request_auth=None,
             _content_type=None,
@@ -156,18 +213,102 @@ class AsyncFileSystem:
             _host_index=None,
         )
 
-        async with httpx.AsyncClient(timeout=timeout or None) as client:
-            async with client.stream(method, url, headers) as response:
-                response.raise_for_status()
-                parent = os.path.dirname(local_path)
-                if parent:
-                    await aiofiles.os.makedirs(parent, exist_ok=True)
+        try:
+            async with httpx.AsyncClient(timeout=timeout) as client:
+                async with client.stream(
+                    method,
+                    url,
+                    json=body,
+                    headers=headers,
+                ) as resp:
+                    resp.raise_for_status()
 
-                async with aiofiles.open(local_path, "wb") as f:
-                    async for chunk in response.aiter_bytes(chunk_size=10 * 1024):
-                        if chunk:
-                            await f.write(chunk)
-            return None
+                    content_type, options = parse_options_header(resp.headers.get("Content-Type", ""))
+                    if not (content_type == "multipart/form-data" and "boundary" in options):
+                        raise DaytonaError(f"Unexpected Content-Type: {content_type}")
+                    boundary = options["boundary"]
+
+                    with PushMultipartParser(boundary) as parser:
+                        writer = None
+                        mode = None  # "file" or "error"
+                        source = None
+
+                        async for chunk in resp.aiter_bytes(64 * 1024):
+                            if parser.closed:
+                                raise DaytonaError("Unexpected end of multipart data")
+
+                            for result in parser.parse(chunk):
+                                if isinstance(result, MultipartSegment):  # New part starting
+                                    writer = None
+                                    mode = None
+                                    source = result.filename
+                                    if not source:
+                                        raise DaytonaError(f"No source path found for this file {result.filename}")
+
+                                    if result.name == "error":
+                                        mode = "error"
+                                    elif result.name == "file":
+                                        mode = "file"
+                                        meta = src_file_meta_dict[source]
+                                        if meta.dst:
+                                            parent = os.path.dirname(meta.dst)
+                                            if parent:
+                                                await aiofiles.os.makedirs(parent, exist_ok=True)
+                                            # pylint: disable=consider-using-with
+                                            writer = await aiofiles.open(meta.dst, mode="wb")
+                                            file_writers.append(writer)
+                                            meta.result = meta.dst
+                                        else:
+                                            writer = io.BytesIO()
+                                            meta.result = writer
+
+                                elif result:  # Non-empty bytearray with content
+                                    if mode == "error":
+                                        error_text = bytes(result).decode("utf-8", errors="ignore").strip()
+                                        src_file_meta_dict[source].error = error_text
+                                    elif mode == "file":
+                                        try:
+                                            if isinstance(writer, io.BytesIO):
+                                                writer.write(bytes(result))
+                                            else:
+                                                await writer.write(bytes(result))
+                                        except Exception as e:
+                                            src_file_meta_dict[source].error = f"Write failed: {e}"
+                                            mode = None
+
+                                else:  # None - end of current part
+                                    if writer and not isinstance(writer, io.BytesIO):
+                                        await writer.close()
+                                    writer = None
+                                    mode = None
+                                    source = None
+        finally:
+            for writer in file_writers:
+                await writer.close()
+
+        # Build results for all requested files
+        results = []
+        for f in files:
+            meta = src_file_meta_dict[f.source]
+            # see if there's an explicit error; if not, but no data, set a default error
+            err = meta.error
+            if not err and not meta.result:
+                err = "No data received for this file"
+            # only fetch the value if there was no error
+            res = None
+            if err is None:
+                res = meta.result
+                if isinstance(res, io.BytesIO):
+                    res = res.getvalue()
+            results.append(
+                FileDownloadResponse(
+                    source=f.source,
+                    result=res,
+                    error=err,
+                )
+            )
+
+        return results
 
     @intercept_errors(message_prefix="Failed to find files: ")
     async def find_files(self, path: str, pattern: str) -> List[Match]:
@@ -176,8 +317,8 @@ class AsyncFileSystem:
 
         Args:
             path (str): Path to the file or directory to search. If the path is a directory,
-                the search will be performed recursively. Relative paths are resolved based on the user's
-                root directory.
+                the search will be performed recursively. Relative paths are resolved based
+                on the sandbox working directory.
             pattern (str): Search pattern to match against file contents.
 
         Returns:
@@ -196,7 +337,7 @@ class AsyncFileSystem:
         """
         return await self._toolbox_api.find_in_files(
             self._sandbox_id,
-            path=prefix_relative_path(await self._get_root_dir(), path),
+            path=path,
             pattern=pattern,
         )
 
@@ -206,8 +347,8 @@ class AsyncFileSystem:
         size, permissions, and timestamps.
 
         Args:
-            path (str): Path to the file or directory. Relative paths are resolved based on the user's
-            root directory.
+            path (str): Path to the file or directory. Relative paths are resolved based
+            on the sandbox working directory.
 
         Returns:
             FileInfo: Detailed file information including:
@@ -234,17 +375,15 @@ class AsyncFileSystem:
                 print("Path is a directory")
             ```
         """
-        return await self._toolbox_api.get_file_info(
-            self._sandbox_id, path=prefix_relative_path(await self._get_root_dir(), path)
-        )
+        return await self._toolbox_api.get_file_info(self._sandbox_id, path=path)
 
     @intercept_errors(message_prefix="Failed to list files: ")
     async def list_files(self, path: str) -> List[FileInfo]:
         """Lists files and directories in a given path and returns their information, similar to the ls -l command.
 
         Args:
-            path (str): Path to the directory to list contents from. Relative paths are resolved based on the user's
-            root directory.
+            path (str): Path to the directory to list contents from. Relative paths are resolved
+            based on the sandbox working directory.
 
         Returns:
             List[FileInfo]: List of file and directory information. Each FileInfo
@@ -265,19 +404,17 @@ class AsyncFileSystem:
             print("Subdirectories:", ", ".join(d.name for d in dirs))
             ```
         """
-        return await self._toolbox_api.list_files(
-            self._sandbox_id, path=prefix_relative_path(await self._get_root_dir(), path)
-        )
+        return await self._toolbox_api.list_files(self._sandbox_id, path=path)
 
     @intercept_errors(message_prefix="Failed to move files: ")
     async def move_files(self, source: str, destination: str) -> None:
         """Moves or renames a file or directory. The parent directory of the destination must exist.
 
         Args:
-            source (str): Path to the source file or directory. Relative paths are resolved based on the user's
-            root directory.
-            destination (str): Path to the destination. Relative paths are resolved based on the user's
-            root directory.
+            source (str): Path to the source file or directory. Relative paths are resolved
+            based on the sandbox working directory.
+            destination (str): Path to the destination. Relative paths are resolved based on
+            the sandbox working directory.
 
         Example:
             ```python
@@ -302,8 +439,8 @@ class AsyncFileSystem:
         """
         await self._toolbox_api.move_file(
             self._sandbox_id,
-            source=prefix_relative_path(await self._get_root_dir(), source),
-            destination=prefix_relative_path(await self._get_root_dir(), destination),
+            source=source,
+            destination=destination,
         )
 
     @intercept_errors(message_prefix="Failed to replace in files: ")
@@ -312,8 +449,7 @@ class AsyncFileSystem:
 
         Args:
             files (List[str]): List of file paths to perform replacements in. Relative paths are
-            resolved based on the user's
-            root directory.
+            resolved based on the sandbox working directory.
             pattern (str): Pattern to search for.
             new_value (str): Text to replace matches with.
 
@@ -342,7 +478,7 @@ class AsyncFileSystem:
             ```
         """
         for i, file in enumerate(files):
-            files[i] = prefix_relative_path(await self._get_root_dir(), file)
+            files[i] = file
 
         replace_request = ReplaceRequest(files=files, new_value=new_value, pattern=pattern)
 
@@ -354,8 +490,8 @@ class AsyncFileSystem:
         specified pattern. The pattern can be a simple string or a glob pattern.
 
         Args:
-            path (str): Path to the root directory to start search from. Relative paths are resolved based on the user's
-            root directory.
+            path (str): Path to the root directory to start search from. Relative paths are resolved
+            based on the sandbox working directory.
             pattern (str): Pattern to match against file names. Supports glob
                 patterns (e.g., "*.py" for Python files).
 
@@ -377,7 +513,7 @@ class AsyncFileSystem:
         """
         return await self._toolbox_api.search_files(
             self._sandbox_id,
-            path=prefix_relative_path(await self._get_root_dir(), path),
+            path=path,
             pattern=pattern,
         )
 
@@ -387,8 +523,8 @@ class AsyncFileSystem:
         to leave that attribute unchanged.
 
         Args:
-            path (str): Path to the file or directory. Relative paths are resolved based on the user's
-            root directory.
+            path (str): Path to the file or directory. Relative paths are resolved based on
+            the sandbox working directory.
             mode (Optional[str]): File mode/permissions in octal format
                 (e.g., "644" for rw-r--r--).
             owner (Optional[str]): User owner of the file.
@@ -412,7 +548,7 @@ class AsyncFileSystem:
         """
         await self._toolbox_api.set_file_permissions(
             self._sandbox_id,
-            path=prefix_relative_path(await self._get_root_dir(), path),
+            path=path,
             mode=mode,
             owner=owner,
             group=group,
@@ -426,8 +562,8 @@ class AsyncFileSystem:
 
         Args:
             file (bytes): File contents as a bytes object.
-            remote_path (str): Path to the destination file. Relative paths are resolved based on the user's
-            root directory.
+            remote_path (str): Path to the destination file. Relative paths are resolved based on
+            the sandbox working directory.
             timeout (int): Timeout for the upload operation in seconds. 0 means no timeout. Default is 30 minutes.
 
         Example:
@@ -459,7 +595,7 @@ class AsyncFileSystem:
         Args:
             local_path (str): Path to the local file to upload.
             remote_path (str): Path to the destination file in the Sandbox. Relative paths are
-            resolved based on the user's root directory.
+            resolved based on the sandbox working directory.
             timeout (int): Timeout for the upload operation in seconds. 0 means no timeout. Default is 30 minutes.
 
         Example:
@@ -504,15 +640,14 @@ class AsyncFileSystem:
 
         with ExitStack() as stack:
             for i, f in enumerate(files):
-                dst = prefix_relative_path(await self._get_root_dir(), f.destination)
-                data_fields[f"files[{i}].path"] = dst
+                data_fields[f"files[{i}].path"] = f.destination
 
                 if isinstance(f.source, (bytes, bytearray)):
                     stream = io.BytesIO(f.source)
-                    filename = dst
+                    filename = f.destination
                 else:
                     stream = stack.enter_context(open(f.source, "rb"))
-                    filename = dst
+                    filename = f.destination
 
                 # HTTPX will stream this file object in 64 KiB chunks :contentReference[oaicite:1]{index=1}
                 file_fields[f"files[{i}].file"] = (filename, stream)
