@@ -14,11 +14,9 @@ from .._utils.stream import process_streaming_response
 from .._utils.timeout import with_timeout
 from ..common.errors import DaytonaError
 from ..common.image import Image
-from ..common.snapshot import CreateSnapshotParams, Snapshot
+from ..common.snapshot import CreateSnapshotParams, PaginatedSnapshots, Snapshot
 from .object_storage import AsyncObjectStorage
 
-SNAPSHOTS_FETCH_LIMIT = 200
-
 
 class AsyncSnapshotService:
     """Service for managing Daytona Snapshots. Can be used to list, get, create and delete Snapshots."""
@@ -28,24 +26,37 @@ class AsyncSnapshotService:
         self.__object_storage_api = object_storage_api
 
     @intercept_errors(message_prefix="Failed to list snapshots: ")
-    async def list(self) -> List[Snapshot]:
-        """List all Snapshots.
+    async def list(self, page: Optional[int] = None, limit: Optional[int] = None) -> PaginatedSnapshots:
+        """Returns paginated list of Snapshots.
+
+        Args:
+            page (Optional[int]): Page number for pagination (starting from 1).
+            limit (Optional[int]): Maximum number of items per page.
 
         Returns:
-            List[Snapshot]: List of all Snapshots.
+            PaginatedSnapshots: Paginated list of Snapshots.
 
         Example:
             ```python
             async with AsyncDaytona() as daytona:
-                snapshots = await daytona.snapshot.list()
-                for snapshot in snapshots:
+                result = await daytona.snapshot.list(page=2, limit=10)
+                for snapshot in result.items:
                     print(f"{snapshot.name} ({snapshot.image_name})")
             ```
         """
-        response = await self.__snapshots_api.get_all_snapshots(limit=SNAPSHOTS_FETCH_LIMIT)
-        if response.total > SNAPSHOTS_FETCH_LIMIT:
-            response = await self.__snapshots_api.get_all_snapshots(limit=response.total)
-        return [Snapshot.from_dto(snapshot) for snapshot in response.items]
+        if page is not None and page < 1:
+            raise DaytonaError("page must be a positive integer")
+
+        if limit is not None and limit < 1:
+            raise DaytonaError("limit must be a positive integer")
+
+        response = await self.__snapshots_api.get_all_snapshots(limit=limit, page=page)
+        return PaginatedSnapshots(
+            items=[Snapshot.from_dto(snapshot) for snapshot in response.items],
+            total=response.total,
+            page=response.page,
+            total_pages=response.total_pages,
+        )
 
     @intercept_errors(message_prefix="Failed to delete snapshot: ")
     async def delete(self, snapshot: Snapshot) -> None:
@@ -188,6 +199,15 @@ class AsyncSnapshotService:
 
         return created_snapshot if isinstance(created_snapshot, Snapshot) else Snapshot.from_dto(created_snapshot)
 
+    async def activate(self, snapshot: Snapshot) -> Snapshot:
+        """Activate a snapshot.
+        Args:
+            snapshot (Snapshot): The Snapshot instance.
+        Returns:
+            Snapshot: The activated Snapshot instance.
+        """
+        return Snapshot.from_dto(await self.__snapshots_api.activate_snapshot(snapshot.id))
+
     @staticmethod
     async def process_image_context(object_storage_api: ObjectStorageApi, image: Image) -> List[str]:
         """Processes the image context by uploading it to object storage.
@@ -201,18 +221,18 @@ class AsyncSnapshotService:
 
         push_access_creds = await object_storage_api.get_push_access()
 
-        async with AsyncObjectStorage(
+        object_storage = AsyncObjectStorage(
             push_access_creds.storage_url,
             push_access_creds.access_key,
             push_access_creds.secret,
             push_access_creds.session_token,
             push_access_creds.bucket,
-        ) as object_storage:
-            context_hashes = []
-            for context in image._context_list:  # pylint: disable=protected-access
-                context_hash = await object_storage.upload(
-                    context.source_path, push_access_creds.organization_id, context.archive_path
-                )
-                context_hashes.append(context_hash)
+        )
+        context_hashes = []
+        for context in image._context_list:  # pylint: disable=protected-access
+            context_hash = await object_storage.upload(
+                context.source_path, push_access_creds.organization_id, context.archive_path
+            )
+            context_hashes.append(context_hash)
 
         return context_hashes
